local string = string
local type = type

function table.copy(source, destiny)
	if source then
		if not destiny then destiny = {} end
		for field, value in pairs(source) do
			rawset(destiny, field, value)
		end
	end
	return destiny
end

local env
function string:tag(tags)
	return ( self:gsub("<(%a+)>", tags or env) )
end

--------------------------------------------------------------------------------

env = table.copy(getfenv(1))

setfenv(1, env)

--------------------------------------------------------------------------------

workdir = "."

OPT = env
TAG = string.tag

local output = io.stdout
local tabs = 0

function TAB() return string.rep("\t", tabs) end
function INC() tabs = tabs + 1 end
function DEC() tabs = tabs - 1 end
function RAW(...) output:write(...) end

function TO(path)
	if output ~= io.stdout then
		output:close()
		output = io.stdout
	end
	output = assert(io.open(workdir.."/"..path, "w"))
end

local CustomTags = { __index = OPT }
local function updtabs(cap, ...) if cap == "{" then INC() else DEC() end end
function OUT(...)
	local tags
	for i = 1, select("#", ...) do
		local value = select(i, ...)
		if value then
			if type(value) == "string" then
				local tabs, close
				tabs, close, value = value:tag(tags):match("^[ \n]*(\t*)(}?)(.-)$")
				if close == "}" then DEC() end
				RAW(TAB(), close, value:gsub("\n"..tabs, "\n"..TAB()), "\n")
				value:gsub("%b{}", ""):gsub("[{}]", updtabs)
			elseif #value == 0 then
				tags = setmetatable(table.copy(value, tags), CustomTags)
			else
				for i, item in ipairs(value) do
					OUT(tags, item)
				end
			end
		end
	end
end

function ARG(...)
	for i = 1, select("#", ...) do
		local arg = select(i, ...)
		local name, value = arg:match("^([%w_]+)=(.+)$")
		if name then
			OPT[name] = tonumber(value) or (value:lower() ~= "no" and value or nil)
		end
	end
end

--------------------------------------------------------------------------------

local i = 0
repeat
	pcall(setfenv, i, OPT)
	i = i + 1
until not debug.getinfo(i)
