require "gencode"


ARG(...)

local imagescale="0.55"

local docdate=os.date("%F")

local reports = {
	"octet",
	"boolean",
	"char",
	"short",
	"ushort",
	"long",
}

local image = [[
\begin{figure}
	\begin{center}
%		\caption{Time spent in invocations that take a parameter of type \texttt{<type>}.}
		\includegraphics[scale=<scale>]{<name>}
	\end{center}
\end{figure}
]]

TO("../output/doc/report.tex")

OUT({date=docdate}, [[
\documentclass[]{report}

% Use utf-8 encoding for foreign characters
\usepackage[utf8]{inputenc}

% Setup for fullpage use
\usepackage{fullpage}

% Surround parts of graphics with box
\usepackage{boxedminipage}

% Package for including inline lists
\usepackage{paralist}

% Package for coloring fonts and backgrounds
\usepackage{color}

% Package for including code in the document
\usepackage{listings}

% If you want to generate a toc for each chapter (use with book)
\usepackage{minitoc}

% This is now the recommended way for checking for PDFLaTeX:
\usepackage{ifpdf}

\ifpdf
\usepackage[pdftex]{graphicx}
\usepackage{epstopdf}
\else
\usepackage{graphicx}
\fi

\title{FTC Performance Tests Report}
\author{Victor Fusco [vfusco@tecgraf.puc-rio.br]}

\date{<date>}

\begin{document}

\maketitle

\ifpdf
\DeclareGraphicsExtensions{.pdf, .jpg, .tif}
\else
\DeclareGraphicsExtensions{.eps, .jpg}
\fi
]])


local imagetags= {name="void",scale=imagescale, type="void"}

OUT([[\chapter{Network and Hard Disk Test Results}]])
--OUT(, image)

OUT( [[\chapter{FTC Implementations Test Results}]])

OUT( [[\chapter{FTC Clients Read Buffer Size Variation Test Results}]])

OUT( [[\chapter{TCP Window Size Test Results}]])
imagetags.name="../images/iperf.eps"
OUT( imagetags, image )

OUT( [[\chapter{Detailed Hard Disk Test Results}]])
imagetags.name="../images/read.eps"
OUT( imagetags , image )
imagetags.name="../images/reread.eps"
OUT( imagetags , image )

OUT( [[\end{document}]] )
