/**
 * $Id$
 */
package ftc;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;
import ftc.client.FTCClientV1_1;
import ftc.client.FTCClientV1_2;
import ftc.client.FTCClientV1_3;
import ftc.client.FTCClientV1_4_0;

/**
 * 
 * 
 * @author Tecgraf
 */
public class FtcClient {
  @Option(name = "-h", usage = "Nome/IP do servidor FTC.")
  private String host = "127.0.0.1";

  @Option(name = "-p", usage = "Porta TCP do servidor FTC")
  private int port = 25555;

  @Option(name = "-b", usage = "Tamanho do buffer")
  private int bufferSize = 1024 * 1024; // 1MB

  @Option(name = "-s", usage = "Quantidade de bytes a ser transferidos")
  private long nbytes = -1;

  @Option(name = "-f", usage = "Nome do arquivo a ser transferido", required = true)
  private String filename = "";

  @Option(name = "-o", usage = "Nome do arquivo a ser escrito localmente")
  private String outputFilename = "/dev/null";

  @Option(name = "-c", usage = "Exibe o resultado em CSV")
  private boolean csv_output = false;

  @Option(name = "-l", usage = "Nome do arquivo de log")
  private String logFilename = "";

  @Option(name = "-u", usage = "Faz upload de um arquivo")
  private boolean upload = false;

  @Option(name = "-ftcversion", usage = "Define a verso do FTC a ser utilizada")
  private FTCVersion ftcVersion = FTCVersion.FTC_V1_1;

  /**
   * Transfere um arquivo em uma maquina remota
   * 
   * @param fileName
   * @param host
   * @param port
   * @throws NoSuchAlgorithmException
   */
  public long getFile() throws NoSuchAlgorithmException {

    MessageDigest key = MessageDigest.getInstance("MD5");
    key.update(filename.getBytes(), 0, filename.length());

    FTCClientVersionAdapter rfc = getClient(filename, host, port, key.digest());
    long size = 0;

    try {
      File file = new File(outputFilename);

      FileOutputStream newfile = new FileOutputStream(file);

      rfc.setBufferSize(bufferSize);

      long t0 = System.currentTimeMillis();
      rfc.open(true);
      long t1 = System.currentTimeMillis();

      // Se o tamanho nao for definido na linha de comando, perguntamos 
      // ao servidor o tamanho atual do arquivo.
      size = nbytes < 0 ? rfc.getSize() : nbytes;

      long nbytes = 0;
      long t2 = System.currentTimeMillis();

      if (rfc.getVersion().equals(FTCVersion.FTC_V1_4_0)) {
        nbytes = rfc.transferTo(0, size, newfile.getChannel());
      }
      else {
        nbytes = rfc.transferTo(0, size, newfile);
      }

      long t3 = System.currentTimeMillis();

      rfc.close();

      double responseTime = (((double) t1) - ((double) t0)) / 1000.0;
      double diff = ((double) t3 - (double) t2) / 1000.0;
      double speed =
        Double.compare(diff, 0.0000000) == 0 ? 0 : (size / 1024) / diff;

      StringBuffer sb = new StringBuffer();

      if (csv_output) {
        sb.append(size + "," + diff + "," + speed + "," + bufferSize + ","
          + responseTime + "," + ((t2) / 1000.0) + "\n");
      }
      else {
        sb.append("Arquivo: " + filename + "\n");
        sb.append("Bytes Transferidos: " + nbytes + "\n");
        sb.append("Tempo decorrido: " + diff + " s\n");
        sb.append("Taxa de transferencia: " + speed + " KB/s\n\n");
      }

      if (logFilename.length() != 0) {
        FileOutputStream logFile = new FileOutputStream(logFilename, true);
        logFile.write(sb.toString().getBytes());
      }
      else {
        System.out.write(sb.toString().getBytes());
      }

    }
    catch (Exception e) {
      e.printStackTrace();
    }

    return size;
  }

  /**
   * Transfere um arquivo local para uma maquina remota.
   * 
   * @param fileName nome do arquivo.
   * @param host Nome/IP da maquina remota.
   * @param port porta do servico.
   * @throws NoSuchAlgorithmException
   */
  public long putFile() throws NoSuchAlgorithmException {

    MessageDigest key = MessageDigest.getInstance("MD5");
    key.update(outputFilename.getBytes(), 0, outputFilename.length());

    FTCClientVersionAdapter rfc =
      getClient(outputFilename, host, port, key.digest());
    long size = 0;

    try {

      File file = new File(filename);
      FileInputStream fileStream = new FileInputStream(file);

      long t0 = System.currentTimeMillis();
      rfc.open(false);
      long t1 = System.currentTimeMillis();

      // Se o tamanho nao for definido na linha de comando, usamos 
      // o tamanho atual do arquivo.
      size = nbytes < 0 ? file.length() : nbytes;

      rfc.setSize(size);

      long transfered = 0;
      long t2 = System.currentTimeMillis();

      if (rfc.getVersion().equals(FTCVersion.FTC_V1_4_0)) {
        transfered = rfc.transferFrom(fileStream.getChannel(), 0, size);
      }
      else {
        transfered = rfc.transferFrom(fileStream, 0, size);
      }

      long t3 = System.currentTimeMillis();

      rfc.close();

      double responseTime = (((double) t1) - ((double) t0)) / 1000.0;
      double diff = ((double) t3 - (double) t2) / 1000.0;
      double speed =
        Double.compare(diff, 0.0000000) == 0 ? 0 : (transfered / 1024) / diff;

      StringBuffer sb = new StringBuffer();

      if (csv_output) {
        sb.append(transfered + "," + diff + "," + speed + "," + bufferSize
          + "," + responseTime + "," + ((t2) / 1000.0) + "\n");
      }
      else {
        sb.append("Arquivo: " + filename + "\n");
        sb.append("Bytes Transferidos: " + transfered + "\n");
        sb.append("Tempo decorrido: " + diff + " s\n");
        sb.append("Taxa de transferencia: " + speed + " KB/s\n\n");
      }

      if (logFilename.length() != 0) {
        FileOutputStream logFile = new FileOutputStream(logFilename, true);
        logFile.write(sb.toString().getBytes());
      }
      else {
        System.out.write(sb.toString().getBytes());
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    return size;
  }

  private FTCClientVersionAdapter getClient(String fileName, String host,
    int port, byte[] key) {

    FTCClientVersionAdapter rfc = null;

    switch (ftcVersion) {
      case FTC_V1_4_1:
        throw new NotImplementedException();
      case FTC_V1_4_0:
        rfc = new FTCClientV1_4_0(fileName, host, port, key);
        break;
      case FTC_V1_3:
        rfc = new FTCClientV1_3(fileName, host, port, key);
        break;
      case FTC_V1_2:
        rfc = new FTCClientV1_2(fileName, host, port, key);
        break;
      case FTC_V1_1:
      default:
        rfc = new FTCClientV1_1(fileName, host, port, key);
        break;
    }

    return rfc;
  }

  /**
   * Print usage
   */
  public static void printUsage() {
    System.out
      .println("java -jar ftcclient.jar -h host [-p port] [-c] -f /full/remote/path/to/file ");
  }

  /**
   * @param args
   * @throws NoSuchAlgorithmException
   */
  public static void main(String[] args) throws NoSuchAlgorithmException {

    FtcClient ftc_client = new FtcClient();

    CmdLineParser parser = new CmdLineParser(ftc_client);

    try {
      parser.parseArgument(args);
    }
    catch (CmdLineException e) {
      printUsage();
      parser.printUsage(System.err);
      System.exit(1);
    }

    if (ftc_client.upload) {
      ftc_client.putFile();
    }
    else {
      ftc_client.getFile();
    }

    System.exit(0);
  }

}
