package ftc.server.v1_1;

import java.io.FileNotFoundException;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.nio.channels.FileChannel;

import tecgraf.ftc.common.exception.FailureException;
import tecgraf.ftc.common.exception.PermissionException;
import tecgraf.ftc.server.FileServerConfig;
import tecgraf.ftc.server.FileServerOwner;
import tecgraf.ftc.utils.Utils;
import ftc.ServerConfig;

public class FTCBenchServerOwner implements FileServerOwner {

  FileServerConfig config = null;

  public FTCBenchServerOwner(ServerConfig c) {
    config = new FtcServerConfig(c);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public FileChannel createFileChannel(Object requester, byte[] fileId,
    boolean readOnly) throws FailureException, PermissionException {
    String absolutePath;
    try {
      absolutePath = new String(fileId, Utils.CHARSET_ENCODING);
    }
    catch (UnsupportedEncodingException e) {
      throw new FailureException(e);
    }

    if (absolutePath.endsWith("NO_PERMISSION")) {
      throw new PermissionException("Sem permisso para abrir o arquivo");
    }

    if (absolutePath.endsWith("NOT_FOUND")) {
      return null;
    }

    String mode = "r";
    if (!readOnly) {
      mode += "w";
    }
    RandomAccessFile raf;
    try {
      raf = new RandomAccessFile(absolutePath, mode);
    }
    catch (SecurityException e) {
      throw new PermissionException(e);
    }
    catch (FileNotFoundException e) {
      return null;
    }
    return raf.getChannel();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void exceptionRaised(Exception e, byte[] fileId) {
    try {
      String absolutePath = new String(fileId, Utils.CHARSET_ENCODING);
      System.err.println(String.format("fileId = %s", absolutePath));
      e.printStackTrace();
    }
    catch (UnsupportedEncodingException e1) {
      e1.printStackTrace();
    }
  }

  @Override
  public void exceptionRaised(Exception e) {
    e.printStackTrace();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void fileChannelClosed(Object requester, byte[] fileId) {
    try {
      String absolutePath = new String(fileId, Utils.CHARSET_ENCODING);
      System.out.println("O arquivo " + absolutePath + " foi fechado.");
    }
    catch (UnsupportedEncodingException e) {
      e.printStackTrace();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isLocked(Object requester, byte[] fileId) {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public FileServerConfig getConfig() {
    return config;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setConfig(FileServerConfig config) {
    if (config == null) {
      return;
    }

    this.config = config;
  }

}
