package.path = "./share/lua/5.1/?.lua;" .. package.path

require("ftc")
require("getopt")

local md5 = require("md5")
local socket = require("socket")
local opts = getopt( arg, "hpblsc" )


-- Parsing command line
local SERVER_HOST =  opts["h"] or "127.0.0.1"
local SERVER_PORT =  opts["p"] or 25555
local FileSize    = (opts["s"] or 0)*1
local BufferSize  = (opts["b"] or 1024*1024*50)*1
local CSV_OUTPUT  =  opts["c"] or false
local LogFilename =  opts["l"] or false
local filename    =  arg[optind]

if not filename then
	print("O nome do aquivo não foi dado." )
	os.exit(1)
end

local null = "/dev/null"
local writable = false
local size = 0


-- Abrindo o arquivo de saida
local outfile ,err = io.open(null,"w")
if not outfile then
	print("Erro ao abrir o arquivo: " .. null)
	print(err)
	os.exit(1)
end


-- Abrindo a conexao para receber o arquivo remoto
local t0 = socket.gettime()
local file = ftc(filename, writable, SERVER_HOST, SERVER_PORT, md5.sum(filename))
assert(file:open(not writable))

-- Definindo o tamanho do arquivo dinamicamente
local size = FileSize
if( size <= 0 ) then
    local status
    status, size = file:getSize()
    if not status then
        print("Erro ao requisitar o tamanho do arquivo: " .. null)
        print(size)
        os.exit(1)
    end
end

file:setReadBufferSize(BufferSize)

local t1 = socket.gettime()

local nbytes = file:transferTo(0,size,outfile)

local t2 = socket.gettime()

file:close()

local totaltime = t2-t1 
local response_time = t1-t0

local speed = (size/1024)/totaltime

local output = io.output

if LogFilename then
    local file ,err = io.open(LogFilename,"a+")
    if not file then
	    print("Erro ao abrir o arquivo: " .. LogFilename)
    	print(err)
    else
        output = file
    end
end

if CSV_OUTPUT then
	output:write( size .. ","..totaltime .. "," .. speed .. "," .. BufferSize .. "," .. response_time .. "," .. t1 .. "\n" )
else
	output:write("File size: " .. size/1024 .. " KB\n" )
	output:write("Tempo decorrido: " .. totaltime .. " s\n")
	output:write("Taxa de transferencia: " .. speed .. " KB/s\n")
	output:write("Tamanho do Buffer: " .. BufferSize .. " Bytes\n")
end

output:close()

