
local io     = io
local print  = print
local ipairs = ipairs
local pairs  = pairs
local PARAMS = ftcbench.parameters

module("ftcbench.java")

local name = "FTC Java Client"

local command  = "java -classpath \"./java/lib/ftcbench.jar:./java/lib/ftc-1.4.0.1.jar:./java/lib/ftc-1.3.1.jar:./java/lib/ftc-1.2.0.jar:./java/lib/ftc-1.1.5.jar:./java/lib/args4j-2.0.9.jar\" ftc.FtcClient -ftcversion FTC_V1_4_0"
local debugcmd = "java -agentlib:hprof=cpu=times -classpath \"./build:./java/lib/ftc-1.4.0.1.jar:./java/lib/ftc-1.3.1.jar:./java/lib/ftc-1.2.0.jar:./java/lib/ftc-1.1.5.jar:./java/lib/args4j-2.0.9.jar\" ftc.FtcClient -ftcversion FTC_V1_4_0"

local conf 

function setConfig(config)
    conf = config
end

function transferFile(filename)
    local logFile = conf.OutputDir .. "java-" .. conf.ServerName .. ".txt"               
    if conf.LogFile ~= nil then
        logFile = conf.OutputDir .. conf.LogFile
    end

    local params = " -h " .. conf.Hostname .. 
                   " -p " .. conf.Port ..
                   " -b " .. conf.BufferSize .. 
                   " -l " .. logFile ..
                   " -c " 

    if conf.FileSize > 0 then
        params = params ..  " -s " .. conf.FileSize .. " "
    end

    local cmd = command 
    if conf.Profile then
        cmd = debugcmd
    end

    local fullcmd =  cmd .. params .. " -f " .. PARAMS.SERVER_FILES_DIR .. filename  
    if conf.Upload then
        fullcmd = cmd .. params .. " -u "
        fullcmd = fullcmd .. " -f " .. PARAMS.LOCAL_FILES_DIR .. filename
        fullcmd = fullcmd .. " -o " .. PARAMS.SERVER_FILES_DIR .. filename .. "-UPLOAD"
    end

    if conf.Debug then
        print("Command: " .. fullcmd)
    end

    local output = io.popen(fullcmd)

    if not output then
        return nil , name .. " Error: Could not execute the given command!"
    end

    output:close()
end

function getName()
    return name
end

function run()
    for i=1,conf.RepeatTimes  do
        for _,file in ipairs(conf.Files) do
            print( "Transfering file: " .. file )
            transferFile(file)
        end
    end
end
