
local io     = io
local print  = print
local ipairs = ipairs
local pairs  = pairs
local PARAMS = ftcbench.parameters

module("ftcbench.m2mjava")

local name = "FTC Java Client"

local command  = "java -classpath \"./java/lib/ftcbench.jar:./java/lib/ftc-1.4.0.1.jar:./java/lib/ftc-1.3.1.jar:./java/lib/ftc-1.2.0.jar:./java/lib/ftc-1.1.5.jar:./java/lib/args4j-2.0.9.jar\" ftc.FtcClient -ftcversion FTC_V1_4_0"
local debugcmd = "java -agentlib:hprof=cpu=times -classpath \"./build:./java/lib/ftc-1.4.0.1.jar:./java/lib/ftc-1.3.1.jar:./java/lib/ftc-1.2.0.jar:./java/lib/ftc-1.1.5.jar:./java/lib/args4j-2.0.9.jar\" ftc.FtcClient -ftcversion FTC_V1_4_0"

local conf 

function setConfig(config)
    conf = config
end

function transferFile(filename,size)
    local logFile = conf.OutputDir .. "m2m-java-" .. conf.ServerName .. ".txt"               
    if conf.LogFile ~= nil then
        logFile = conf.OutputDir .. conf.LogFile
    end

    local opts = " -h " .. conf.Hostname .. 
                 " -p " .. conf.Port ..
                 " -b " .. conf.BufferSize .. 
                 " -s " .. size .. 
                 " -l " .. logFile ..
                 " -c "

    local cmd = command 
    if conf.Profile then
        cmd = debugcmd
    end

    local fullcmd =  cmd .. opts .. " -f " .. filename
    if conf.Upload then
        fullcmd = cmd .. opts .. " -u "
        fullcmd = fullcmd .. " -f " .. filename
        fullcmd = fullcmd .. " -o " .. "/dev/null"
    end
    if conf.Debug then
        print("Command: " .. fullcmd)
    end

    local output = io.popen(fullcmd)

    if not output then
        return nil , name .. " Error: Could not execute the given command!"
    end

    output:close()
    return true
end

function getName()
    return name
end

function run()
    for i=1,#PARAMS.M2M_SIZES do
        local size = PARAMS.M2M_SIZES[i]
        for i=1,conf.RepeatTimes  do
            print( "Transfering "..size .." bytes from file: " .. PARAMS.MEM2MEM_FILE )
            local status,msg = transferFile( PARAMS.MEM2MEM_FILE , size )
            if not status then
                print(msg)
                break
            end
        end
    end
end
